//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

#include <tigcclib.h>         // Include All Header Files

#include "cardslib.h"
#include "extgraph.h"

//---------------------------------------------------------------------------

char * SuitStr[] = {"SPADE","HEART","DIAMONDS","CLUBS"};
char * RankStr[] = {"A","2","3","4","5","6","7","8","9","10","V","D","R"};

extern char * Plane0;

TCardGame * CardGame;
TCardPack * MainCardPack;

//---------------------------------------------------------------------------

unsigned short Image_C0_FRONT[] = {0xFFFF,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0xFFFF};
unsigned short Image_C0_BACK[] = {0xFFFF,0x8001,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0xAAA9,0x9555,0x8001,0xFFFF};
unsigned short Image_C0_MASK[] = {0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000};

unsigned short Image_C0_SUITS[] = {0x0800,0x1C00,0x3E00,0x7F00,0xFF80,0xFF80,0xFF80,0x6B00,0x1C00,0x1C00,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x7700,0xF780,0xFF80,0xFF80,0xFF80,0x7F00,0x3E00,0x1C00,0x0800,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x1000,0x3800,0x7C00,0xFE00,0xFE00,0xFE00,0x7C00,0x3800,0x1000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0E00,0x1F00,0x1F00,0x1F00,0x6EC0,0xFFE0,0xFFE0,0xFFE0,0x75C0,0x0E00,0x0E00,0x0000,0x0000,0x0000,0x0000,0x0000};

//---------------------------------------------------------------------------

void CreateCards(void)
{
	int a;
	
	
	for (a=0;a<CARDS_COUNT;a++)
	{
		TCard*CardPtr=&CardGame->Cards[a];
		
		CardPtr->Suit = (a/13);
		CardPtr->Rank = (a%13)+1;
		CardPtr->IsFaceVisible = 0;
		CardPtr->Position.x = 0;
		CardPtr->Position.y = 0;
	}
	
}

//---------------------------------------------------------------------------

void CardPackCollect(TCardPack * CardPack)
{
	
	CardPack->Count=CARDS_COUNT;
	
	int a;
	for (a=0;a<CARDS_COUNT;a++)
	{
		TCard*CardPtr=&CardGame->Cards[a];
		
		CardPack->Cards[a]=CardPtr;
		CardPtr->CardPackOwner=CardPack;
	}
	
	// HIDE CARDS FACES
	for (a=0;a<CARDS_COUNT;a++)
	CardPack->Cards[a]->IsFaceVisible=0;
	
}
//---------------------------------------------------------------------------

void CardPackClear(TCardPack * CardPack)
{
	CardPack->Count = 0;
}
//---------------------------------------------------------------------------

void CardPackRandom(TCardPack * CardPack)
{
	int a;
	int RandomPlace;
	
	for (a=0;a<RANDOM_PACK_COUNT;a++)
	{
		while (!(RandomPlace = random(CardPack->Count)));
		ExchangeCards(CardPack->Cards[0],CardPack->Cards[RandomPlace]);
	}
	
}
//---------------------------------------------------------------------------

void CardPackSort(TCardPack * CardPack)
{
	int a,b;
	
	for (a=1;a<CardPack->Count;a++)
	{
		
		b = a;
		
		for (;;)
		{
			if (b==0 || (CardPack->Cards[b]>CardPack->Cards[b-1]))
			break;
			
			ExchangeCards(CardPack->Cards[b],CardPack->Cards[b-1]);
			b--;
		}
		
	}
	
}
//---------------------------------------------------------------------------

void CardPackAlign(TCardPack * CardPack)
{
	
	int a;
	
	for (a=0;a<CardPack->Count;a++)
	{
		
		TCard * CardPtr = CardPack->Cards[a];
		CardPtr->Position = CardPack->InitialPos;
		
		if (CardPack->OrderType==TO_LINE)
		{
			
			switch (CardPack->LineOrderWay)
			{
				case PLAYER_D : CardPtr->Position.x+=(a*CardPack->OrderStep);break;
				case PLAYER_L : CardPtr->Position.y+=(a*CardPack->OrderStep);break;
				case PLAYER_U : CardPtr->Position.x-=(a*CardPack->OrderStep);break;
				case PLAYER_R : CardPtr->Position.y-=(a*CardPack->OrderStep);break;
			}
			
		}
		else if (CardPack->OrderType==TO_CIRCLE)
		{
			
			if (CardPack==MainCardPack)
			CardPtr->CircleOrderPlace=(a&3);
			
			switch (CardPtr->CircleOrderPlace)
			{
				case PLAYER_D : CardPtr->Position.y+=CardPack->OrderStep;break;
				case PLAYER_L : CardPtr->Position.x-=CardPack->OrderStep;break;
				case PLAYER_U : CardPtr->Position.y-=CardPack->OrderStep;break;
				case PLAYER_R : CardPtr->Position.x+=CardPack->OrderStep;break;
			}
			
			
		}
	}
	
}
//---------------------------------------------------------------------------


void CardPackFlip(TCardPack * CardPack)
{
	
	int a;
	for (a=0;a<CardPack->Count;a++)
	CardPack->Cards[a]->IsFaceVisible^=1;
	
}
//---------------------------------------------------------------------------

void CardMovePack(TCard * Card, TCardPack * CardPackDest, int IndexDest)
{
	
	int a;
	TCardPack * CardPackSrc = Card->CardPackOwner;
	
	// 	IF SAME CARDPACK -> RETURN
	if (CardPackSrc==CardPackDest)
	return;
	// IF CARDPACK FULL -> RETURN
	if (CardPackDest->Count==CARDS_COUNT)
	return;
	// IF INDEX>END -> INDEX=END
	if (IndexDest>CardPackDest->Count)
	IndexDest=CardPackDest->Count;
	
	
	// REMOVES CARD FROM SCR
	
	int IndexSrc=CardGetIndex(Card);
	
	for (a=IndexSrc;a!=(CardPackSrc->Count-1);a++)
	CardPackSrc->Cards[a] = CardPackSrc->Cards[a+1];
	
	CardPackSrc->Count--;
	
	// ADDS CARD TO DEST
	
	CardPackDest->Count++;
	
	for (a=(CardPackDest->Count-1);a!=IndexDest;a--)
	CardPackDest->Cards[a] = CardPackDest->Cards[a-1];
	
	CardPackDest->Cards[IndexDest] = Card;
	Card->CardPackOwner=CardPackDest;
	
	
}
//---------------------------------------------------------------------------

int CardGetIndex(TCard * Card)
{
	TCardPack*CardPack=Card->CardPackOwner;
	
	int a;
	for (a=0;a<CardPack->Count;a++)
	if (CardPack->Cards[a]==Card)
	return a;
	
	return (-1);
}
//---------------------------------------------------------------------------

void ExchangeCards(TCard * Card1, TCard * Card2)
{
	
	TCardPack*CardPack=Card1->CardPackOwner;
	int Index1=CardGetIndex(Card1);
	int Index2=CardGetIndex(Card2);
	
	TCard*CardPush = CardPack->Cards[Index1];
	CardPack->Cards[Index1] = CardPack->Cards[Index2];
	CardPack->Cards[Index2] = CardPush;
}
//---------------------------------------------------------------------------

void DrawCard(TCard * Card)
{
	
	// CARD ON SCREEN
	if (!(Card->Position.x>=0 && Card->Position.y>=0
	&& Card->Position.x<=(160-CARD_WIDTH) && Card->Position.y<=(100-CARD_HEIGHT)))
	return;
	
	unsigned short *ptr = Card->IsFaceVisible?Image_C0_FRONT:Image_C0_BACK;
	Sprite16_MASK(Card->Position.x,Card->Position.y,CARD_HEIGHT,ptr,Image_C0_MASK,Plane0);
	
	if (Card->IsFaceVisible)
	{
		
		// RANK
		DrawStr(
		Card->Position.x+1,
		Card->Position.y+2,
		RankStr[Card->Rank-1],
		A_NORMAL);
		
		// SUIT
		Sprite16_OR(
		Card->Position.x+2,
		Card->Position.y+11,
		16,Image_C0_SUITS+(Card->Suit<<4),
		Plane0);
		
	}
	
	
}
//---------------------------------------------------------------------------

void DrawAllCards(void)
{
	
	int a;
	for (a=0;a<CARDS_COUNT;a++)
	DrawCard(&CardGame->Cards[a]);
	
}
//---------------------------------------------------------------------------

void DrawCardPack(TCardPack * CardPack)
{
	int a;
	for (a=0;a<CardPack->Count;a++)
	DrawCard(CardPack->Cards[a]);
}
//---------------------------------------------------------------------------

