//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 11/02/03; 12:25:49

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "cardslib.h"
#include "menulib.h"
#include "crazy8.h"



//---------------------------------------------------------------------------

extern TCardGame * CardGame;
extern TCardPack * MainCardPack;

//---------------------------------------------------------------------------

extern TMenu * Menu;
enum {ME_MAIN,ME_PAUSE,ME_OPTIONS};

//---------------------------------------------------------------------------

char * Plane0;
TGame *Game;


//---------------------------------------------------------------------------


void NewGame(void)
{
	int a,b;
	
	
	Game->AutoPlay=0;
	
	// MAKE CARDPACKS
	
	CardPackClear(Game->DiscardPile);
	
	for (a=0;a<PLAYERS_COUNT;a++)
	CardPackClear(Game->Players[a].CardPack);
	
	CardPackCollect(MainCardPack);
	CardPackRandom(MainCardPack);
	
	// DEAL OUT CARDS
	
	for (b=0;b<CARS_PER_PLAYER;b++)
	for (a=0;a<PLAYERS_COUNT;a++)
	{
		CardMovePack(MainCardPack->Cards[0],Game->Players[a].CardPack,0);
		
		PlayerMakePos(a);
		CardPackAlign(MainCardPack);
		CardPackAlign(Game->Players[a].CardPack);
		
		RefreshScreen();
	}
	
	// SHOW MAINPLAYER CARDS
	CardPackFlip(Game->Players[MAIN_PLAYER].CardPack);
	
	Game->PlayerTurn = Game->StartPlayer++;
	Game->StartPlayer&=3;
	
}
//---------------------------------------------------------------------------


int TestCard(TCard * Card)
{
	
	if (Game->DiscardPile->Count)
	{
		TCard * LastCard = Game->DiscardPile->Cards[Game->DiscardPile->Count-1];
		return (LastCard->Rank==Card->Rank || LastCard->Suit==Card->Suit);
	}
	else
	return 1;
	
}
//---------------------------------------------------------------------------


void PlayCard(TCard * Card)
{
	
	Card->IsFaceVisible = 1;
	Card->CircleOrderPlace = Game->PlayerTurn;
	
	CardMovePack(Card,Game->DiscardPile,Game->DiscardPile->Count);
	
	PlayerMakePos(Game->PlayerTurn);
	CardPackAlign(Game->Players[Game->PlayerTurn].CardPack);
	CardPackAlign(Game->DiscardPile);
	
	
}
//---------------------------------------------------------------------------

TCard * GetPossiblePlayCard(TCardPack * CardPack)
{
	
	int a;
	for (a=0;a<CardPack->Count;a++)
	{
		TCard*CardPtr = CardPack->Cards[a];
		if (TestCard(CardPtr))
		return CardPtr;
	}
	
	return (NULL);
	
}
//---------------------------------------------------------------------------

void SkipTurn(void)
{
	
	//char buffer[256];
	//sprintf(buffer,"%s : skips his turn.",Game->Players[Game->PlayerTurn].name);
	//ShowMsg(buffer);
	
	DrawFromStock();
	
}
//---------------------------------------------------------------------------

void DrawFromStock(void)
{
	
	
	// MainCardPack EMPTY
	if (MainCardPack->Count==0)
	{
		char buffer[256];
		sprintf(buffer,"%s : cannot draw from stock.",Game->Players[Game->PlayerTurn].name);
		ShowMsg(buffer);
		
		return;
	}
	
	
	TCard * CardPtr = MainCardPack->Cards[0];
	
	if (Game->PlayerTurn==MAIN_PLAYER)
	CardPtr->IsFaceVisible = 1;
	
	CardMovePack(CardPtr,Game->Players[Game->PlayerTurn].CardPack,0);
	
	PlayerMakePos(Game->PlayerTurn);
	CardPackAlign(MainCardPack);
	CardPackAlign(Game->Players[Game->PlayerTurn].CardPack);
	
	if (MainCardPack->Count==0)
	RemakeStock();
	
}
//---------------------------------------------------------------------------



void RemakeStock(void)
{
	
	RefreshScreen();
	ShowMsg("Must remake stock.");
	
	if (Game->DiscardPile->Count>1)
	{
		
		for (;;)
		{
			TCard * CardPtr = Game->DiscardPile->Cards[0];
			CardPtr->IsFaceVisible = 0;
			
			CardMovePack(CardPtr,MainCardPack,0);
			
			CardPackAlign(Game->DiscardPile);
			CardPackAlign(MainCardPack);
			
			if (Game->DiscardPile->Count==1)
			break;
			
		}
		
		CardPackRandom(MainCardPack);
		
	}
	else
	{
		ShowMsg("Cannot remake stock.");
	}
	
}
//---------------------------------------------------------------------------


void CPU_Play(void)
{
	
	TCard * CardPtr=GetPossiblePlayCard(Game->Players[Game->PlayerTurn].CardPack);
	
	if (CardPtr)
	PlayCard(CardPtr);
	else
	SkipTurn();
	
}
//---------------------------------------------------------------------------

void PlayerMakePos(int Player)
{
	
	TCardPack * CardPack = Game->Players[Player].CardPack;
	
	CardPack->OrderStep = Player==MAIN_PLAYER?CARDSTEP_USER:CARDSTEP_CPU;
	
	int center_width =  (160- (CARD_WIDTH+CardPack->Count*CardPack->OrderStep))>>1;
	int center_height = (100-(CARD_HEIGHT+CardPack->Count*CardPack->OrderStep))>>1;
	
	switch (Player)
	{
		
		case PLAYER_D:
		CardPack->InitialPos.x = center_width;
		CardPack->InitialPos.y = (100-CARD_HEIGHT);
		break;
		
		case PLAYER_L:
		CardPack->InitialPos.x = 0;
		CardPack->InitialPos.y = center_height;
		break;
		
		case PLAYER_U:
		CardPack->InitialPos.x = center_width+((CardPack->Count-1)*CardPack->OrderStep);
		CardPack->InitialPos.y = 0;
		break;
		
		case PLAYER_R:
		CardPack->InitialPos.x = (160-CARD_WIDTH);
		CardPack->InitialPos.y = center_height+((CardPack->Count-1)*CardPack->OrderStep);
		break;
		
	}
	
	
}
//---------------------------------------------------------------------------


void CheckGameState(void)
{
	
	if (Game->Players[Game->PlayerTurn].CardPack->Count==0)
	{
		
		char buffer[256];
		sprintf(buffer,"%s won!",Game->Players[Game->PlayerTurn].name);
		ShowMsg(buffer);
		
		Game->Players[Game->PlayerTurn].Points++;
		ShowPoints();
		
		NewGame();
		
	}
	else
	{
		Game->PlayerTurn++;
		Game->PlayerTurn%=PLAYERS_COUNT;
	}
	
	
	
}
//---------------------------------------------------------------------------


void SetPositions(void)
{
	int a;
	
	// OrderType
	
	for (a=0;a<PLAYERS_COUNT;a++)
	{
		Game->Players[a].CardPack->OrderType = TO_LINE;
		Game->Players[a].CardPack->LineOrderWay=a;
	}
	
	Game->DiscardPile->OrderType = TO_CIRCLE;
	Game->DiscardPile->OrderStep = 8;
	
	MainCardPack->OrderType = TO_CIRCLE;
	MainCardPack->OrderStep = 2;
	
	// Positions
	
	MainCardPack->InitialPos.x = 50;
	MainCardPack->InitialPos.y = (100-CARD_HEIGHT)/2;
	
	Game->DiscardPile->InitialPos.x = 100;
	Game->DiscardPile->InitialPos.y = (100-CARD_HEIGHT)/2;
	
	
}
//---------------------------------------------------------------------------

void RefreshScreen(void)
{
	int a;
	
	PortSet(Plane0,239,127);
	ClrScr();
	
	for (a=0;a<PLAYERS_COUNT;a++)
	DrawCardPack(Game->Players[a].CardPack);
	
	DrawCardPack(Game->DiscardPile);
	
	DrawCardPack(MainCardPack);
	
	PortRestore();
	FastCopyScreen(Plane0,LCD_MEM);
	
}
//---------------------------------------------------------------------------

void ShowMsg(char*str)
{
	
	FontSetSys(0);
	DrawStr(0,0,str,A_REPLACE);
	ngetchx();
	FontSetSys(1);
	
}
//---------------------------------------------------------------------------

void ShowPoints(void)
{
	
	clrscr();
	printf("Points \n\n");
	
	int a;
	for (a=0;a<PLAYERS_COUNT;a++)
	printf("%8s : %03d Points\n",Game->Players[a].name,Game->Players[a].Points);
	
	ngetchx();
	
}
//---------------------------------------------------------------------------

// Main Function
void _main(void)
{
	int a;
	int key;
	
	
	if (!(Plane0=malloc(LCD_SIZE)))
	goto end_prog;
	
	TMenu MenuA;
	Menu = &MenuA;
	TGame GameA;
	Game=&GameA;
	
	FontSetSys(1);
	randomize();
	
	// CARDS ALLOC
	
	TCardGame CardGameA;
	CardGame = &CardGameA;
	
	TCardPack MainCardPackA;
	MainCardPack=&MainCardPackA;
	TCardPack DiscardPileA;
	Game->DiscardPile=&DiscardPileA;
	
	TCardPack Players_CardPacks[PLAYERS_COUNT];
	for (a=0;a<PLAYERS_COUNT;a++)
	Game->Players[a].CardPack=&Players_CardPacks[a];
	
	CreateCards();
	
	strcpy(Game->Players[PLAYER_D].name,"You");
	strcpy(Game->Players[PLAYER_L].name,"CPU2");
	strcpy(Game->Players[PLAYER_U].name,"CPU3");
	strcpy(Game->Players[PLAYER_R].name,"CPU4");
	
	for (;;)
	{
		MenuNew2();
		MenuAddTitle("Crazy Eights");
		MenuAddItem("Play");
		MenuAddItem("About");
		MenuAddItem("Quit");
		
		if (!MenuDo(ME_MAIN))
		break;
		
		
		if (Menu->Index==1)
		{
			
			// RESET POINTS
			for (a=0;a<PLAYERS_COUNT;a++)
			Game->Players[a].Points=0;
			
			Game->StartPlayer=MAIN_PLAYER;
			
			SetPositions();
			NewGame();
			RefreshScreen();
			
			
			for (;;)
			{
				
				
				
				autoplay_lbl:
				
				if (Game->PlayerTurn==MAIN_PLAYER && !Game->AutoPlay)
				{
					
					TCardPack*MainPack=Game->Players[MAIN_PLAYER].CardPack;
					int CardIndex=MainPack->Count/2;
					
					for (;;)
					{
						
						
						TCard*CurrentCard=MainPack->Cards[CardIndex];
						
						CurrentCard->Position.y-=5;
						RefreshScreen();
						
						key = ngetchx();
						
						CurrentCard->Position.y+=5;
						RefreshScreen();
						
						
						if (key==KEY_LEFT)
						{
							CardIndex--;
							if (CardIndex<0)
							CardIndex = (MainPack->Count-1);
						}
						
						else if (key==KEY_RIGHT)
						{
							CardIndex++;
							if (CardIndex>(MainPack->Count-1))
							CardIndex = 0;
						}
						
						else if (key==KEY_F1)
						{
							if (TestCard(CurrentCard))
							{
								PlayCard(CurrentCard);
								break;
							}
						}
						
						else if (key==KEY_F2)
						{
							SkipTurn();
							break;
						}
						
						else if (key==KEY_F3)
						{
							
							TCard*CardPtr=GetPossiblePlayCard(MainPack);
							if (CardPtr)
							CardIndex=CardGetIndex(CardPtr);
							else
							{
								SkipTurn();
								break;
							}
							
						}
						
						else if (key==KEY_F4)
						{
							CardPackSort(MainPack);
							CardPackAlign(MainPack);
						}
						
						else if (key==KEY_F5)
						{
							Game->AutoPlay=1;
							goto autoplay_lbl;
						}
						
						else if (key==KEY_APPS)
						{
							off();
						}
						
						else if (key==KEY_ESC)
						{
							clrscr();
							printf("[ESC] : Continue \n[ENTER] : Exit");
							while ((key = ngetchx())!=KEY_ENTER && key!=KEY_ESC);
							
							if (key==KEY_ENTER)
							goto end_loop;
						}
						
					}
					
				}
				else
				{
					CPU_Play();        
				}
				
				RefreshScreen();
				CheckGameState();
				
				
				
			}
			
			end_loop:
			
			
		}
		else if (Menu->Index==2)
		{
			
			FontSetSys(0);
			clrscr();
			printf("About \n");
			printf("Crazy Eights 1.2 \n");
			printf("Programmed by Olivier Sangala \n");
			printf("http://olivier.sangala.free.fr \n");
			printf("olivier_sangala@hotmail.com \n");
			FontSetSys(1);
			ngetchx();
			
		}
		else if (Menu->Index==3)
		{
			break;
		}
		
		
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
}
